import httpx

from .exceptions import HttpException


class BaseAPI:

    timeout = 10

    def __init__(self, symbol=None):
        """
        Constructor.

        :param str symbol: Symbol of te item we wanna get info about.
        """

        if symbol:
            self.symbol = str(symbol)

    def _get(self, *args):
        """
        TBD
        """

        try:
            return httpx.get(*args, timeout=self.timeout)
        except Exception as exc:
            raise HttpException(
                f"Couldn't perform GET request with args {args}"
            ) from exc

    async def _aget(self, *args):

        async with httpx.AsyncClient() as client:

            try:
                return await client.get(*args, timeout=self.timeout)
            except Exception as exc:
                raise HttpException(
                    f"Couldn't perform GET request with args {args}"
                ) from exc



from . import BaseAPI


class UnofficialAPI(BaseAPI):
    def get_current_price(self):
        """
        Fetches current market price inc. pre/post market
        prices/percent/value changes. Also returns current
        market state (pre-market, open, post-market).

        Fetched stucture has following fields:

        * state (pre-market, open, post-market)
        * pre_market
            * change
                * percents
                * value
            * value
        * current_market
            * change
                * percents
                * value
            * value
        * post_market
            * change
                * percents
                * value
            * value

        Returned dict looks like:

        .. code-block:: python

            {
                "state": "open",
                "pre_market": {
                    "change": {
                        "percents": -1.32476,
                        "value": -1.42001
                    },
                    "value": 105.77
                },
                "current_market": {
                    "change": {
                        "percents": -1.6046284000000002,
                        "value": -1.7200012
                    },
                    "value": 105.47
                },
                "post_market": {
                    "change": {
                        "percents": 0.0,
                        "value": 0.0
                    },
                    "value": 0.0
                }
            }

        :return: Current/Pre/Post market numbers (all are floats).
        :rtype: dict
        """

        def parse_state(state):

            # No state at all.
            if not state:
                return None

            state = state.lower()

            if state.startswith("pre"):
                return "pre-market"

            if state.startswith("post"):
                return "post-market"

            return "open"

        response = self._get(
            f"https://query1.finance.yahoo.com/v11/finance/quoteSummary/{self.symbol}?modules=price"
        )
        data = response.json()
        data = data["quoteSummary"]["result"][0]["price"]

        return {
            "state": parse_state(data.get("marketState")),
            "pre_market": {
                "change": {
                    "percents": float(
                        data.get("preMarketChangePercent", {}).get("raw", 0)
                    )
                    * 100,
                    "value": float(data.get("preMarketChange", {}).get("raw", 0)),
                },
                "value": float(data.get("preMarketPrice", {}).get("raw", 0)),
            },
            "current_market": {
                "change": {
                    "percents": float(
                        data.get("regularMarketChangePercent", {}).get("raw", 0)
                    )
                    * 100,
                    "value": float(data.get("regularMarketChange", {}).get("raw", 0)),
                },
                "value": float(data.get("regularMarketPrice", {}).get("raw", 0)),
            },
            "post_market": {
                "change": {
                    "percents": float(
                        data.get("postMarketChangePercent", {}).get("raw", 0)
                    )
                    * 100,
                    "value": float(data.get("postMarketChange", {}).get("raw", 0)),
                },
                "value": float(data.get("postMarketPrice", {}).get("raw", 0)),
            },
        }



import asyncio
import time
from datetime import datetime

from . import BaseAPI


class UnofficialAPI(BaseAPI):
    """
    Unofficial API wrapper class for TipRanks.com.
    Unofficial means this class calls some hidden endpoints
    and provides data that official API doesn't. Also doesn't
    need an authorization.
    """

    def get_dividends(self):
        """
        Fetches symbol dividends with following fields:

        * yield
        * amount
        * payment_date
        * ex_date
        * receive_date
        * growth_since

        :return: List of dividend objects.
        :rtype: list
        """

        response = self._get(
            f"https://tr-frontend-cdn.azureedge.net/bff/prod/stock/{self.symbol.lower()}/payload.json",
        )
        data = response.json()

        dividends = []

        if data["dividend"]["history"]:
            for item in data["dividend"]["history"]:
                dividends.append(
                    {
                        "yield": float(item["yield"]),
                        "amount": float(item["amount"]),
                        "payment_date": datetime.strptime(
                            item["payDate"], "%Y-%m-%dT%H:%M:%S.000Z"
                        ).date(),
                        "ex_date": datetime.strptime(
                            item["executionDate"], "%Y-%m-%dT%H:%M:%S.000Z"
                        ).date(),
                        "receive_date": datetime.strptime(
                            item["recordDate"], "%Y-%m-%dT%H:%M:%S.000Z"
                        ).date(),
                        "growth_since": datetime.strptime(
                            item["growthSince"], "%Y-%m-%dT%H:%M:%S.000Z"
                        ),
                    }
                )

        return dividends

    def get_basic_info(self):
        """
        Downloads basic info. Data are:

        * company_name
        * market
        * description
        * market_cap
        * has_dividends
        * yoy_change
        * year_low
        * year_high
        * pe_ratio
        * eps
        * similar_stocks
            * ticker
            * company_name

        :return: Dict with data.
        :rtype: dict
        """

        async def download_basics(symbol):
            """
            Downloads basic info about symbol. Data are:

            * company_name
            * market
            * description
            * market_cap
            * has_dividends
            * similar_stocks
                * ticker
                * company_name
            * yoy_change

            :param str symbol: Symbol the data will be downloaded for.
            :return: Dict with data.
            :rtype: dict
            """

            data = await self._aget(
                f"https://tr-frontend-cdn.azureedge.net/bff/prod/stock/{symbol.lower()}/payload.json"
            )
            json_data = data.json()

            data = {
                "company_name": json_data["common"]["stock"]["fullName"],
                "market": json_data["common"]["stock"]["market"],
                "description": json_data["common"]["stock"]["description"],
                "market_cap": int(json_data["common"]["stock"]["marketCap"]),
                "has_dividends": bool(
                    json_data["common"]["stock"]["dividend"].get("yield", False)
                ),
                "yoy_change": float(
                    json_data["common"]["stock"]["technical"]["momentum"][
                        "twelveMonths"
                    ]
                    or 0.0
                )
                * 100,
                "year_low": json_data["common"]["stock"]["priceRange"]["last52Week"][
                    "low"
                ],
                "year_high": json_data["common"]["stock"]["priceRange"]["last52Week"][
                    "high"
                ],
                "pe_ratio": json_data["common"]["stock"]["pe"],
                "eps": json_data["common"]["stock"]["earning"]["espYear"],
                "similar_stocks": [],
            }

            if json_data["similar"]["similar"]:
                for stock in json_data["similar"]["similar"]:
                    data["similar_stocks"].append(
                        {"ticker": stock["ticker"], "company_name": stock["name"]}
                    )

            return data

        async def download_additionals(symbol):
            """
            Downloads additional info. Data are:

            * year_low
            * year_high
            * pe_ratio
            * eps

            :param str symbol: Symbol the data will be downloaded for.
            :return: Dict with data.
            :rtype: dict
            """

            data = await self._aget(
                f"https://market.tipranks.com/api/details/GetRealTimeQuotes/?tickers={symbol}"
            )
            json_data = data.json()

            return {
                "year_low": json_data[0]["yLow"],
                "year_high": json_data[0]["yHigh"],
                "pe_ratio": json_data[0]["pe"],
                "eps": json_data[0]["eps"],
            }

        async def main():

            basic, additionals = await asyncio.gather(
                download_basics(self.symbol), download_additionals(self.symbol)
            )
            basic.update(additionals)

            return basic

        return asyncio.run(main())



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rug']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.11.1,<0.12.0']

setup_kwargs = {
    'name': 'rug',
    'version': '0.2.6',
    'description': 'Library for fetching various stock data from the internet (official and unofficial APIs).',
    'long_description': '# Rug\n\nUniversal library for fetching Stock data from the internet - mostly unofficial\nAPIs - no limits, more free data.\n\n(for Cryptocurrency alternative see [karpet](https://github.com/im-n1/karpet))\n\n* [PyPI](https://pypi.org/project/rug/)\n* [documentation](https://rug.readthedocs.io/en/latest/) ![Documentation Status](https://readthedocs.org/projects/rug/badge/?version=latest)\n\n## Changelog\n\n### 0.2.5\n\n- Fixed TipTanks API - basic info for companies with no dividends\n\n### 0.2.4\n\n- Fixed TipRanks API - dividends for companies with no dividends\n\n### 0.2.3\n\n- Fixed TipRanks API - all methods\n\n### 0.2.2\n\n* Minor fixes.\n\n### 0.2.1\n\nMethod `rug.yahoo.UnofficialAPI.get_current_price()` returns market state now.\n\n### 0.2\n\nNew portals added: YAHOO! + StockTwits\n\n* `get_current_price()` method added\n* `get_earnings_calendar` method added\n\n### 0.1.2\n* `get_dividends()` now returns dividend `amount` too\n\n### 0.1.1\n* dates are now `datetime.date` instance\n\n### 0.1\n* initial release\n',
    'author': 'Pavel Hrdina, Patrick Roach',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/im-n1/rug',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import time
from datetime import date, timedelta

from . import BaseAPI


class UnofficialAPI(BaseAPI):
    def get_earnings_calendar(self, from_date, to_date):
        """
        Fetches upcomming earnings annoucements within
        the given date range.
        Maximum date span is 14 days.

        Each earning dict has the following structure:

        * date
        * symbol
        * time
        * when (BEFORE_OPEN or AFTER_CLOSE)

        :param date from_date: Beginning of the calendar span.
        :param date end_date: End of the calendar span.
        :return: List of earning objects.
        :rtype: list
        """

        # Check params.
        if not isinstance(from_date, date) or not isinstance(to_date, date):
            raise AttributeError(
                'Both "from_date" and "to_date" params'
                "need to be datetime.date instances."
            )

        # Check date span.
        if 14 < (to_date - from_date).days:
            raise ValueError("Maximum date span is 14 days.")

        response = self._get(
            "https://api.stocktwits.com/api/2/discover/earnings_calendar?"
            f"date_from={from_date}&date_to={to_date}"
        )
        earnings = response.json()["earnings"]
        data = []

        for _, item in earnings.items():
            for stock in item["stocks"]:
                data.append(
                    {
                        "date": date(*time.strptime(stock["date"], "%Y-%m-%d")[:3]),
                        "symbol": stock["symbol"],
                        "time": time.strptime(stock["time"], "%H:%M:%S"),
                        "when": "BEFORE_OPEN"
                        if "08:00:00" == stock["time"]
                        else "AFTER_CLOSE",
                    }
                )

        return data
